#ifndef __XORCOM_LICENSING_H__
#define __XORCOM_LICENSING_H__

#ifdef __cplusplus

#include <cstddef>
#include <cstdint>

extern "C" {

#else // __cplusplus

#include <stddef.h>
#include <stdint.h>

#endif // __cplusplus

#ifndef XLIC_LICENSE_FILE
#define XLIC_LICENSE_FILE "/etc/cpbx/license.bin"
#endif // XLIC_LICENSE_FILE

#ifndef XLIC_LICENSE_SERVER
#define XLIC_LICENSE_SERVER "http://license.xorcom.com/"
#endif // XLIC_LICENSE_SERVER

typedef enum {
	XLIC_INVALID_LICENSE,
	XLIC_VIRTUAL_LICENSE,
	XLIC_PHYSICAL_LICENSE
} xlic_license_type;

struct __attribute__((__packed__)) xlic_public_key {
	uint8_t data[94];
};

struct __attribute__((__packed__)) xlic_license_key {
	uint8_t data[20];
};

struct __attribute__((__packed__)) xlic_signature {
	uint8_t data[64];
};

struct xlic_license {
	xlic_license_type type;
	char * name;
	char * description;
	uint64_t generated;
	uint32_t expires;
	struct xlic_license_key key;
	struct xlic_signature signature;
};

struct xlic_license_list {
	size_t count;
	struct xlic_public_key public_key;
	struct xlic_license licenses[0];
};

extern int xlic_list_licenses(
	struct xlic_license_list ** list,
	const char * filename);

extern void xlic_destroy_license_list(struct xlic_license_list * list);

extern int xlic_register_license(
	const char * filename,
	const char * url,
	const char * license_key,
	const char * serial_number);

extern int xlic_update_licenses(
	const char * filename,
	const char * url);

extern int xlic_remove_license(
	const char * filename,
	const char * url,
	const char * license_key);

extern int xlic_find_product_license(
	struct xlic_license ** license,
	const char * filename,
	const char * product_name);

extern void xlic_destroy_license(struct xlic_license * license);

extern xlic_license_type xlic_license_key_type(const char * license_key);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // __XORCOM_LICENSING_H__
